!{\src2tex{textfont=tt}}
!!****f* USPP2PAW/defs_pawps
!! NAME
!! defs_pawps
!!
!! FUNCTION
!! This module contains the following structured data types:
!!   pawps_type   : Specific paw variables
!!   pshead_type  : psp file header variables
!!   pawrad_type  : Radial mesh definition
!!   usdata_type  : Various data inherited from uspp code
!!   pawopt_type  : options for computation
!!
!! COPYRIGHT
!! Copyright (C) 2005-2014 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the GNU General Public License
!! See http://www.gnu.org/copyleft/gpl.txt
!!
!! NOTES
!! The paw wavefunctions tphi and phi, and the projectors tprojector radial
!! parts are defined as: f(r_vector)=f(r)/r* Ylm(r_vector)
!!
!! SOURCE

 module defs_pawps

 use defs_basis

 implicit none


!Structures
!**********************************************************************

 type pawps_type

! ===== Real (double precision) arrays

  real(dp), pointer :: factscale(:)
   ! factscale(basis_size)
   ! Scaling factors for Phi, tPhi and  tProj.

  real(dp), pointer :: phi(:,:)
   ! phi(wav_meshsz,basis_size)
   ! Gives, on the radial grid, the atomic partial waves

  real(dp), pointer :: tphi(:,:)
   ! tphi(wav_meshsz,basis_size)
   ! Gives, on the radial grid, the atomic pseudo partial waves

  real(dp), pointer :: tproj(:,:)
   ! tproj(prj_msz_max,basis_size)
   ! Gives, on the radial grid, the projectors on partial waves

  real(dp), pointer :: coredens(:)
   ! coredens(core_meshsz)
   ! Gives the core density of the atom

  real(dp), pointer :: coreden4pr2(:)
   ! coreden4pr2(max_meshsz)
   ! Gives the core density multiplied by 4Pi.r2 (read from uspp)

  real(dp), pointer :: tcoredens(:)
   ! tcoredens(core_meshsz)
   ! Gives the pseudo core density of the atom

  real(dp), pointer :: tcoreden4pr2(:)
   ! tcoreden4pr2(max_meshsz)
   ! Gives the pseudo core density multiplied by 4Pi.r2 (read from uspp)

  real(dp), pointer :: valeden4pr2(:)
   ! valeden4pr2(max_meshsz)
   ! Gives the valence density multiplied by 4Pi.r2 (read from uspp)

  real(dp), pointer :: tvaledens(:)
   ! tvaledens(vale_meshsz)
   ! Gives the pseudo valence density of the atom

  real(dp), pointer :: tvaleden4pr2(:)
   ! tvaleden4pr2(max_meshsz)
   ! Gives the pseudo valence density multiplied by 4Pi.r2 (read from uspp)

  real(dp), pointer :: ushatden4pr2(:)
   ! ushatden4pr2(max_meshsz)
   ! Gives the US compensation density (from uspp) multiplied by 4Pi.r2

  real(dp), pointer :: dij0(:)
   ! dij0(lmn2_size)
   ! Part of the Dij term (non-local operator) completely
   !  calculated in the pseudopotential part

  real(dp), pointer :: kij(:)
   ! kij(lmn2_size)
   ! Kinetic overlap operator

  real(dp), pointer :: rhoij0(:)
   ! rhoij0(lmn2_size)
   ! Atomic initialization of rhoij

  real(dp), pointer :: shapefunc(:)
   ! shapefunc(wav_meshsz)
   ! Gives the normalized shape function
   ! used for the compensation density

  real(dp), pointer :: shpf(:,:)
   !shapefunction (mesh_size,l_size)
   ! Gives the normalized l-dependent shape function
   ! used for the compensation density

  real(dp), pointer :: shpnrm(:)
   ! shpnrm(0:l_size)
   ! Gives the normalisation of shapefunction for each l
   ! shpnrm(0) is the norm of hat density

  real(dp), pointer :: vbare(:)
   ! vbare(wav_meshsz)
   ! Gives the vbare local potential
   ! used in Blochl/Holzwarth's formulation
   ! but not in Kress's formulation of VH(tnzc)

  real(dp), pointer :: vhtnzc(:)
   ! vhtnzc(max_meshsz)
   ! Gives the Hartree potential of the pseudo density
   !  of the nucleus + core electrons of the atom

  real(dp), pointer :: rvloc0(:)
   ! rvloc0(max_meshsz)
   ! Gives the effective descreeened pseudized atomic potential*r

 end type pawps_type

! =====================================================================

 type pshead_type

! ===== Integer scalars

  integer :: basis_size
   ! Number of elements for the paw nl basis

  integer :: core_meshsz
   ! Dimension of radial mesh for core and tcore densities

  integer :: creatorid
   ! ID of psp generator (here creatorID=1 !)

  integer :: l_size
   ! Maximum value of l+1 leading to a non zero Gaunt coefficient

  integer :: lambda          ! Lambda in gaussian type g(r)

  integer :: lmax
   ! Maximum value of l

  integer :: lmn_size
   ! Number of elements for the paw basis

  integer :: lmn2_size
   ! lmn2_size=lmn_size*(lmn_size+1)/2

  integer :: max_meshsz
   ! Maximal mesh size for densities/potentials

  integer :: mesh_type
   ! Flag: 1 if all meshes are regular grids
   !       2 if all meshes are log. grids

  integer :: nmesh
   ! Maximal number of meshes

  integer :: prj_meshsz
   ! Dimension of radial mesh for tproj

  integer :: prj_msz_max
   ! Maximal dimension of radial mesh for tproj

  integer :: pspcod
   ! Psp code number for Abinit (here PAW->pspcod=7 !)

  character*8 :: pspdat
   ! Psp date creation

  integer :: pspversion    ! Version of the ABINIT PAW proprietary format

  integer :: pspxc_abinit
   ! ABINIT code number for the exchange-correlation functionnal

  integer :: shape_type
   ! Shape function type
   ! shape_type=-1; g(r)=tabulated
   ! shape_type=1 ; g(r)=sin
   ! shape_type=2 ; g(r)=exp

  integer :: sph_meshsz
   ! Dimension of radial mesh for spheres

  integer :: vale_meshsz
   ! Dimension of radial mesh for vale and tvale densities

  integer :: vbar_meshsz
   ! Dimension of radial mesh for Vbare
   ! For r>r(Vbare), we assume VAE=Vloc

  integer :: vloc_meshsz
   ! Dimension of radial mesh for vloc=vhtnzc
   ! For r>r(vloc_meshsz), we assume vloc=-Z/r

  integer :: vlocopt
   ! Option for the format of Vloc
   !  1: Vloc is VH(tnzc) (Kresse s formulation)
   !  2: Vloc is VH(tnzc) (Blochl s formulation without compensation in XC)

  integer :: wav_meshsz
   ! Mesh size for partial waves and projectors inherited from uspp

! ===== Integer arrays

   integer, pointer :: indlmn(:,:)
   ! indlmn(6,lmn_size)
   ! Gives l,m,n,lm,ln,s for i=lmn
   integer, pointer :: orbitals(:)
   ! orbitals(basis_size)
   ! Gives the quantum number l for each basis function

! ===== Real (double precision) scalars

  real(dp) :: atomic_charge
   ! Total atomic charge

  real(dp) :: core_charge
   ! Core charge

  real(dp) :: log_step
   ! Logarithimic step corresponding to radial mesh

  real(dp) :: rad_step
   ! Radial step corresponding to radial mesh

  real(dp) :: rc_sph
   ! Default PAW sphere radius

  real(dp) :: rc_hat
   ! Default PAW sphere radius

  real(dp) :: sigma
   ! Sigma for gaussian shape function

  real(dp) :: valence_charge
   ! Valence charge

! ===== Strings

  character*(fnlen) :: title
   ! Title for pseudopotential

 end type pshead_type

! =====================================================================

 type pawrad_type

! ===== Integer scalars
  integer :: mesh_type

  integer :: mesh_size
   ! Max. dimension of radial meshes

  integer :: int_meshsz              ! Mesh size used in integrals computation
                                     ! Integrals will be computed up to r(int_meshsz)
! ===== Real scalars

  real(dp) :: lstep
   ! Logarithimic step corresponding to radial mesh

  real(dp) :: rstep
   ! Radial step corresponding to radial mesh

  real(dp) :: stepint                ! Radial step used to convert any function from the
                                     ! present grid onto a regular grid in order to
                                     ! integrate it using trapeze method
! ===== Real (double precision)

! ===== Real (double precision) arrays

  real(dp), pointer :: rad(:)
   ! rad(mesh_size)
   ! Coordinates of all the points of the radial grid

  real(dp), pointer :: radfact(:)
   ! radfact(mesh_size)
   ! radfact(:)= factor used to compute radial integrals on generalized grid

  real(dp), pointer :: simfact(:)
   ! simfact(mesh_size)
   ! Factor used to compute radial integrals by the a Simpson scheme
   ! Integral[f] = Sum_i [simfact(i)*f(i)]

 end type pawrad_type

! =====================================================================

 type usdata_type

! ===== Integer scalars

  integer :: nval
   ! Number of valence states

! ===== Real (double precision) scalars

  real(dp) :: exfact
   ! USpp s code number for the exchange-correlation functionnal

! ===== Integer arrays

   integer, pointer :: keyee(:)
   ! keyee(basis_size)
   ! indice of reference eigenenergy for a given basis function
   ! (0 if the the reference energy in not an eigenenergy)

   integer, pointer :: lval(:)
   ! lval(nval)
   ! l angular momentum for each valence state

! ===== Real (double precision) arrays

  real(dp), pointer :: aesnl(:,:)
   ! aesnl(max_meshsz,nval)
   ! Eigenfunctions of AE Hamiltonian

  real(dp), pointer :: dion(:,:)
   ! dion(basis_size,basis_size)
   ! Descreened atomic Dij quantities

  real(dp), pointer :: eig(:)
   ! eig(basis_size)
   ! Eigen- or reference energy for each partial wave

  real(dp), pointer :: occ(:)
   ! occ(nval)
   ! Valence states occupancies inherited from USPP

  real(dp), pointer :: qijps(:,:,:)
   ! phi(wav_meshsz,basis_size,basis_size)
   ! Gives, on the radial grid, pseudized qij quantities,
   ! i.e. phi_i(r)*phi_j(r)-tphi_i(r)*tphi_j(r)

  real(dp), pointer :: pawsnl(:,:)
   ! pawsnl(max_meshsz,nval)
   ! Eigenfunctions of US/PAW Hamiltonian

  real(dp), pointer :: snlprj(:,:)
   ! snlprj(basis_size,nval)
   ! Projected eigentstates, i.e. <PAWSnl(j)|Proj(i)> scalars

  real(dp), pointer :: rvae(:)
   ! rvae(max_meshsz)
   ! Gives the atomic AE potential*r

  real(dp), pointer :: rvloc(:)
   ! rvloc(max_meshsz)
   ! Gives the pseudized atomic potential*r

  real(dp), pointer :: rvloc0(:)
   ! rvloc0(max_meshsz)
   ! Gives the descreeened pseudized atomic potential*r
   ! (descreened inside USPP)

  real(dp), pointer :: rsvale(:)
   ! rsvale(max_meshsz)
   ! Gives the (pseudo valence density + hat density) multiplied by 4Pi.r2 (read from uspp)

 end type usdata_type

! =====================================================================

 type pawrso_type

! ===== Logical

  logical :: userso
   ! TRUE if Real Space Optimization is required

! ===== Real (double precision)

  real(dp) :: ecut
   ! Real Space Optimization parameter: plane wave cutoff = 1/2 Gmax**2

  real(dp) :: gfact
   ! Real Space Optimization parameter: Gamma/Gmax ratio

  real(dp) :: werror
   ! Real Space Optimization parameter: max. error W_l allowed

 end type pawrso_type

! =====================================================================

 type pawopt_type

! ===== Integer scalars

  integer :: idescr
   !  Option governing the descreening of pseudized potential
   !  idescr= 0: descreened pseudized potential is taken from USPP
   !          1: pseudized potential is descreened here

  integer :: iortho
   !  Option governing the orthogonalization of Phi, tPhi and tProj
   !  iortho= 0: Phi, tPhi and tProj are not orthogonalized inside USPP
   !          1: Phi, tPhi and tProj are orthogonalized inside USPP

  integer :: iscalephi
   !  Option governing the normalization of Phi, tPhi and tProj
   !  iscalephi= 0: Phi, tPhi and tProj from USPP are unchanged
   !             1: Phi, tPhi and tProj from USPP are normalized so that:
   !                  bound states are equal to atomic PAW Hamiltonian eigenfunctions,
   !                  unbound states are normalized in PAW sphere
   !             2: Phi, tPhi and tProj from USPP are normalized so that:
   !                  bound states are equal to atomic PAW Hamiltonian eigenfunctions,
   !                  unbound states are equal to 1 at PAW sphere boundary

  integer :: ishapenum
   !  Option governing the writing of shape functions
   !  ishapenum= 0: shape functions are written as analytic (only if ishapetyp>0)
   !             1: shape functions are written as numeric

  integer :: ishapetyp
   !  Option governing the type of the shape function g(r)
   !  ishapetyp= 0: k(r) is read from uspp, g(r)=N.k(r).r^l
   !             1: k(r) is [sin(r/rc)/(r/rc)]**2 with rc=rcompfact*rcut, g(r)=N.k(r).r^l
   !             1: g(r)=[a1.jl(q1.r)+a2.jl(q2.r)] with g(rc)=0 at rc=rcompfact*rcut

  integer :: ivbare
   !  Option governing the value of "bare" potential
   !  ivbare= 0: vbare is forced to be zero
   !          1: vbare is computed

  integer :: usexcnhat
   !  Option governing the the use of compensation charge density in
   !  eXchange-Correlation terms (Kresse's formulation) or not (Blochl's formulation)
   !  usexcnhat= 0: nhat not used in XC
   !             1: nhat used in XC

! ===== Real (double precision)

  real(dp) :: rcompfact
   ! Define radius for compensation charge (rc_hat=rcompfact*rcut)
   ! shape_function(r>=rc_hat)=0

 end type pawopt_type

!**********************************************************************

 end module defs_pawps
!!***
