 !{\src2tex{textfont=tt}}
!!****f* USPP2PAW/wrabinitps
!! NAME
!! wrabinitps
!!
!! FUNCTION
!! write the paw data file in ABINIT proprietary format
!!
!! COPYRIGHT
!! Copyright (C) 2014-2014 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the GNU General Public License
!! See http://www.gnu.org/copyleft/gpl.txt
!!
!! INPUT
!!  fname=name of the psp file
!!  mesh_indx(:)= indexes the different meshes
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %basis_size= Number of elements for the paw nl basis
!!    %core_charge= Core charge
!!    %vlocopt=Option for the format of Vloc
!!              0: Vloc is Vbare (Blochl formulation)
!!              1: Vloc is VH(tnzc) (Kresse formulation)
!!              2: Vloc is VH(tnzc) (Kresse formulation without compensation in XC)
!!    %lambda= Lambda in gaussian type g(r)
!!    %lmn_size= Number of elements for the paw basis
!!    %lmn2_size=lmn_size*(lmn_size+1)/2
!!    %l_size = Maximum value of l+1 leading to a non zero Gaunt coef
!!    %mesh_size   = max. number of radial meshes
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %pspxc_abinit= ABINIT code number for the exchange-correlation
!!    %pspversion= Version of the PAW format for Abinit
!!    %rc_hat= radius for shape function
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type
!!    %sigma= Sigma for gaussian type g(r)
!!    %title= Title for pseudopotential
!!  pawps
!!   %coreden4pr2(:)  ! Gives the core density multiplied by 4Pi.r2
!!   %tcoreden4pr2(:) ! Gives the pseudized core density multiplied by 4Pi.r2
!!   %tvaleden4pr2(:) ! Gives the pseudized core density multiplied by 4Pi.r2 (up to r(vale_meshsz))
!!   %phi(:,:)        ! Gives, on the radial grid, the PAW atomic wavefunctions
!!   %tphi(:,:)       ! Gives, on the radial grid, the PAW atomic pseudowavefunctions
!!   %tproj(:,:)      ! Gives, on the radial grid, the PAW projectors for each basis function
!!   %dij0(:)         ! Part of the Dij term (non-local operator) frozen during SCF
!!   %shpf(:,:)       !shapefunction (mesh_size,l_size)
!!   %vhtnzc(:)       ! Gives the Hartree potential of the pseudo density
!!                    ! of the nucleus + core electrons of the atom
!!  radmesh(:)= datatypes of all the meshes
!!
!! NOTES
!!  File format of formatted PAW psp input for Abinit:
!!  --------------------------------------------------
!!  (1) title (character) line
!!  (2) znucl, zion, pspdat
!!  (3) pspcod, pspxc, lmax, lloc, mmax, r2well
!!  (4) psp_version, creatorID (here, creatorID=2xxx)
!!  (5) basis_size, lmn_size
!!  (6) orbitals (for l=1 to basis_size)
!!  (7) number_of_meshes
!!  For imsh=1 to number_of_meshes
!!      (8)  mesh_index, mesh_type ,sph_meshsz, rad_step[, log_step]
!!  (9) r_cut(SPH)
!!  (10) shape_type, r_shape
!!  For iln=1 to basis_size
!!      (11)  comment(character)
!!      (12) phi(r) (for ir=1 to sph_meshsz(SPH))
!!  For iln=1 to basis_size
!!      (13) comment(character)
!!      (14) tphi(r) (for ir=1 to sph_meshsz(SPH))
!!  For iln=1 to basis_size
!!      (15) comment(character)
!!      (16) tproj(r) (for ir=1 to proj_meshsz(PROJ))
!!  (17) comment(character)
!!  (18) core_density (for ir=1 to core_meshsz(SPH))
!!  (19) comment(character)
!!  (20) tcore_density (for ir=1 to core_meshsz(SPH))
!!  (21) comment(character)
!!  (22) Dij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!!  (23) comment(character)
!!  (24) Rhoij0 (for ij=1 to lmn_size*(lmn_size+1)/2)
!!  (25) comment(character)
!!  (26) VHntzc(r) (=Vloc) (for ir=1 to vloc_meshsz(VLOC))
!!  ===== Following lines only if shape_type=-1 =====
!!  For il=1 to 2*max(orbitals)+1
!!      (27) comment(character)
!!      (28) radial mesh index for shapefunc
!!      (29) shapefunc(r)/shpnrm(l)*r**l (for ir=1 to phi_meshsz)
!!  --------------------------------------------------
!!
!! PARENTS
!!  wrpawps
!!
!! CHILDREN
!! 
!! SOURCE

 subroutine wrabinitps(fname,mesh_indx,pawps,pshead,radmesh)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
!Scalars
 integer, parameter :: nmesh_max=6
 character*(fnlen) :: fname
 type(pawps_type)  :: pawps
 type(pshead_type) :: pshead
!Arrays
 integer :: mesh_indx(nmesh_max)
 type(pawrad_type) :: radmesh(nmesh_max)

!Local variables ---------------------------------------
!Scalars
 integer, parameter :: ab_unit=13
 integer :: ii,ib,ir,il,ilmn,jlmn,meshsz
 integer :: iwavmsh,iprjmsh,icormsh,ivlcmsh,ishpfmsh,ivalmsh
 character(len=4) :: pspfmt
 character*8 :: strdate
!Arrays

! *************************************************************************

!Format version
 write(unit=pspfmt,fmt='("paw",i1)') pshead%pspversion

!Radial meshes data
 iwavmsh=mesh_indx(1);iprjmsh=mesh_indx(2);icormsh=mesh_indx(3)
 ivlcmsh=mesh_indx(4);ishpfmsh=mesh_indx(5);ivalmsh=mesh_indx(6)

!Open the file for writing
 open(unit=ab_unit,file=trim(fname)//'.abinit',form='formatted',status='unknown')

!Write the header
 call date_and_time(strdate)
 pshead%pspdat=strdate
 write(ab_unit,'(a)') trim(pshead%title)
 write(ab_unit,'(1x,f7.3,1x,f7.3,1x,a,14x,a)') &
&      pshead%atomic_charge,pshead%valence_charge,&
&      trim(strdate),&
&      " : zatom,zion,pspdat"
 write (ab_unit,'(3(1x,i2)," 0 ",i5," 0.",19x,a)') &
&      pshead%pspcod,&
&      pshead%pspxc_abinit,&
&      pshead%lmax,&
&      radmesh(1)%mesh_size,&
&      " : pspcod,pspxc,lmax,lloc,mmax,r2well"
 write (ab_unit,'(1x,a4,1x,i4,29x,a)') &
&       pspfmt,pshead%creatorid,&
&       " : pspfmt,creatorID"
 write (ab_unit,'(2(1x,i2),33x,a)') &
&       pshead%basis_size,&
&       pshead%lmn_size,&
&       " : basis_size,lmn_size"
 do ib=1,pshead%basis_size
  write (ab_unit,'(1x,i1)',ADVANCE='NO') pshead%orbitals(ib)
 enddo
 if (pshead%basis_size<20) then
  do ib=pshead%basis_size+1,20
   write (ab_unit,'(a)',ADVANCE='NO') '  '
  enddo
 endif
 write (ab_unit,'(a)') ": orbitals"
 write (ab_unit,'(1x,i1,37x,a)') &
&       pshead%nmesh," : number_of_meshes"
 do ii=1,pshead%nmesh
  write (ab_unit,'(1x,i1,1x,i1,1x,i4,1x,es22.16,1x,es22.16,a,i1,a)') &
&        ii,radmesh(ii)%mesh_type,radmesh(ii)%mesh_size,radmesh(ii)%rstep,radmesh(ii)%lstep,&
&           " : mesh ",ii,", type,size,rad_step[,log_step]"
 enddo
 write (ab_unit,'(1x,f13.10,25x,a)') &
&       pshead%rc_sph,&
&       " : r_cut(PAW)"
 if (abs(pshead%rc_hat-pshead%rc_sph)<1.d-10) then
  write (ab_unit,'(i2,a,34x,a)') &
&        pshead%shape_type," 0.",&
&        " : shape_type,rshape"
 else
  write (ab_unit,'(i2,1x,f13.10,23x,a)') &
&        pshead%shape_type,pshead%rc_hat,&
&        " : shape_type(SIN),rshape"
 end if

!Write partial waves and pseudo partial waves
 do ib=1,pshead%basis_size
  write(ab_unit,'(a,i1,a)') "===== PHI ",ib,&
&      " =====   [phi(r)=PHI(r)/r*Ylm(th,ph)]"
  write(ab_unit,'(1x,i1,a)') mesh_indx(iwavmsh),"  : radial mesh index"
  write(ab_unit,'(3(1x,es23.16))') &
&      (pawps%phi(ir,ib),ir=1,radmesh(iwavmsh)%mesh_size)
 enddo
 do ib=1,pshead%basis_size
  write(ab_unit,'(a,i1,a)') "===== TPHI ",ib,&
&      " =====   [tphi(r)=TPHI(r)/r*Ylm(th,ph)]"
  write(ab_unit,'(a)') " 1  : radial mesh index"
  write(ab_unit,'(3(1x,es23.16))') &
&      (pawps%tphi(ir,ib),ir=1,radmesh(iwavmsh)%mesh_size)
 enddo

!Write projectors
 do ib=1,pshead%basis_size
  write(ab_unit,'(a,i1,a)') "===== TPROJECTOR ",ib,&
&      " =====   [tp(r)=TPROJECTOR(r)/r*Ylm(th,ph)]"
  write(ab_unit,'(1x,i1,a)') iprjmsh,"  : radial mesh index"
  write(ab_unit,'(3(1x,es23.16))') &
&      (pawps%tproj(ir,ib),ir=1,radmesh(iprjmsh)%mesh_size)
 enddo

!Write the core density and the pseudo core density
 write(ab_unit,'(a)') "===== CORE_DENSITY ====="
 write(ab_unit,'(1x,i1,a)') icormsh,"  : radial mesh index"
 write(ab_unit,'(3(1x,es23.16))') &
&     (pawps%coredens(ir),ir=1,radmesh(icormsh)%mesh_size)
 write(ab_unit,'(a)') "===== PSEUDO_CORE_DENSITY ====="
 write(ab_unit,'(1x,i1,a)') icormsh,"  : radial mesh index"
 write(ab_unit,'(3(1x,es23.16))') &
&     (pawps%tcoredens(ir),ir=1,radmesh(icormsh)%mesh_size)

!Write Dij0 and Rhoij0
 write(ab_unit,'(a)') "===== Dij0 ====="
 ii=0
 do jlmn=1,pshead%lmn_size
  write(ab_unit,'(100(1x,es23.16))') &
&     (pawps%dij0(ii+ilmn),ilmn=1,jlmn)
  ii=ii+jlmn
 enddo
 write(ab_unit,'(a)') "===== Rhoij0 ====="
 ii=0
 do jlmn=1,pshead%lmn_size
  write(ab_unit,'(100(1x,es23.16))') &
&     (pawps%rhoij0(ii+ilmn),ilmn=1,jlmn)
  ii=ii+jlmn
 enddo

!Write VHntZC=Vloc
 if (pshead%vlocopt==1) write(ab_unit,'(a)') "===== VHntZC (Vloc(r)) with compensation charge in XC ====="
 if (pshead%vlocopt==2) write(ab_unit,'(a)') "===== VHntZC (Vloc(r)) without compensation charge in XC ====="
 write(ab_unit,'(i2,1x,i2,a)') ivlcmsh,pshead%vlocopt, &
&   "  : radial mesh index, Vloc format (0=Vbare, 1=VH(tnzc) with hat in XC, 2=VH(tnzc) w/o hat in XC)"
 write(ab_unit,'(3(1x,es23.16))') &
&     (pawps%vhtnzc(ir),ir=1,radmesh(ivlcmsh)%mesh_size)

!Write (eventually) shape functions
 meshsz=radmesh(ishpfmsh)%mesh_size
 if (pshead%shape_type==-1) then
  do il=1,pshead%l_size
   write(ab_unit,'(a,i1,a)') "===== SHAPEF (l=",il-1,") ====="
   write(ab_unit,'(1x,i1,a)') ishpfmsh,"  : radial mesh index"
   write(ab_unit,'(3(1x,es23.16))') (pawps%shpf(ir,il),ir=1,meshsz)
  enddo
 endif

!Write pseudo valence density
 if (pshead%vale_meshsz>0) then
  write(ab_unit,'(a)') "===== PSEUDO_VALENCE_DENSITY ====="
  write(ab_unit,'(1x,i1,a)') ivalmsh,"  : radial mesh index"
  write(ab_unit,'(3(1x,es23.16))') &
&      (pawps%tvaledens(ir),ir=1,radmesh(ivalmsh)%mesh_size)
 end if

!Close the file
 close(ab_unit)

 end subroutine wrabinitps
!!***
