!{\src2tex{textfont=tt}}
!!****f* USPP2PAW/wrpawps
!! NAME
!! wrpawps
!!
!! FUNCTION
!! Write a PAW pseudopotential file formatted for Abinit
!!
!! COPYRIGHT
!! Copyright (C) 1998-2014 ABINIT group (FJ, MT)
!! This file is distributed under the terms of the GNU General Public License
!! See http://www.gnu.org/copyleft/gpl.txt
!!
!! INPUTS
!!  filename= output file name for Abinit
!!  funit= output unit number
!!  pawrad=datastructure containing mesh size definitions
!!  pawps
!!    %coredens(core_meshsz)= Core density of the atom
!!    %tcoredens(core_meshsz)= Pseudo-core density of the atom
!!    %tvaledens(core_meshsz)= Pseudo-valence density of the atom
!!    %dij0(lmn2_size)= Part of the Dij term calculated in the psp part
!!    %phi(wav_meshsz,basis_size)= atomic partial waves
!!    %rc_hat= Radius for compensation charge
!!    %shapefunc(wav_meshsz)= Normalized shape function
!!    %shpnrm(l_size)= Normalisation of shape function for each l
!!    %tphi(wav_meshsz,basis_size)= atomic pseudo partial waves
!!    %tproj(wav_meshsz,basis_size)= projectors on partial waves
!!    %rhoij0= Atomic initialization of rhoij
!!    %vhtnzc(max_meshsz)= Hartree potential of the ps-density
!!                         of the nucleus + core electrons
!!  pshead
!!    %atomic_charge= Total atomic charge
!!    %basis_size= Number of elements for the paw nl basis
!!    %core_meshsz= Dimension of radial mesh for core and tcore densities
!!    %creatorid= ID of psp generator (here creatorID=1 !)
!!    %l_size= Max. value of l+1 leading to a non zero Gaunt coeffs
!!    %lmax= Maximum value of l
!!    %lmn_size= Number of elements for the paw basis
!!    %log_step= Logarithmic step corresponding to radial mesh
!!    %mesh_type=  Flag defining ther radial grid type
!!    %orbitals(basis_size)= Quantum number l for each basis function
!!    %prj_meshsz= Dimension of radial mesh for tProj
!!    %pspcod= Psp code number for Abinit (here PAW->pspcod=7 !)
!!    %pspxc_abinit= Abinit s code number for the exchange-correlation
!!    %rad_step= Radial step corresponding to radial mesh
!!    %rc_sph= Default PAW sphere radius
!!    %shape_type= Shape function type (necessarily 2 here)
!!    %sph_meshsz= Dimension of radial mesh for spheres
!!    %title= Title for pseudopotential
!!    %valence_charge= Valence charge
!!    %vale_meshsz= Dimension of radial mesh for vale and tvale densities
!!    %vloc_meshsz= Dimension of radial mesh for vloc=vhtnzc
!!    %vlocopt= option for the formulation of Vloc
!!    %wav_meshsz=Mesh size for partial waves and projectors inherited from uspp
!!
!! PARENTS
!!      uspp2paw
!!
!! CHILDREN
!!      date_and_time
!!
!! SOURCE

 subroutine wrpawps(fname,pawps,pawrad,pshead)

 use defs_basis
 use defs_pawps

 implicit none

!Arguments ---------------------------------------------
 character*(fnlen) :: fname
 type(pawrad_type)   :: pawrad
 type(pawps_type)    :: pawps
 type(pshead_type)   :: pshead

!Local variables ---------------------------------------
 integer, parameter :: nmesh_max=6,compat46=.false.
 integer :: icormsh,iprjmsh,ivalmsh,ivlcmsh,ii,nmesh,outmeshsz
 integer :: meshsz(nmesh_max), mesh_indx(nmesh_max)
 type(pawrad_type)   :: radmesh(nmesh_max)

! *************************************************************************

!Size of mesh used for functions output
 outmeshsz=pshead%wav_meshsz
!Old coding
!if (     compat46) outmeshsz=min(pshead%sph_meshsz+5,pshead%wav_meshsz)
!if (.not.compat46) outmeshsz=min(pshead%sph_meshsz,pshead%wav_meshsz)
 nmesh=1;meshsz(1)=outmeshsz

!If tproj use a different mesh, adjust flag iprjmsh
 if (pshead%prj_meshsz==outmeshsz) then
  iprjmsh=1
 else
  nmesh=nmesh+1;iprjmsh=nmesh
  meshsz(nmesh)=pshead%prj_meshsz
 end if

!If (t)coredens use a different mesh, adjust flag icormsh
 if (pshead%core_meshsz<=outmeshsz) then
  icormsh=1
 elseif (pshead%core_meshsz==pshead%prj_meshsz) then
  icormsh=iprjmsh
 else
  nmesh=nmesh+1;icormsh=nmesh
  meshsz(nmesh)=pshead%core_meshsz
 endif

!If Vloc use a different mesh, adjust flag ivlcmsh
 if (pshead%vloc_meshsz==outmeshsz) then
  ivlcmsh=1
 elseif (pshead%vloc_meshsz==pshead%prj_meshsz) then
  ivlcmsh=iprjmsh
 elseif (pshead%vloc_meshsz==pshead%core_meshsz) then
  ivlcmsh=icormsh
 else
  nmesh=nmesh+1;ivlcmsh=nmesh
  meshsz(nmesh)=pshead%vloc_meshsz
 endif

!If (t)valedens use a different mesh, adjust flag ivalmsh
 if (pshead%vale_meshsz<=outmeshsz) then
  ivalmsh=1
 elseif (pshead%vale_meshsz==pshead%prj_meshsz) then
  ivalmsh=iprjmsh
 elseif (pshead%vale_meshsz==pshead%core_meshsz) then
  ivalmsh=icormsh
 elseif (pshead%vale_meshsz==pshead%vloc_meshsz) then
  ivalmsh=ivlcmsh
  else
  nmesh=nmesh+1;ivalmsh=nmesh
  meshsz(nmesh)=pshead%vale_meshsz
 endif

!Create array of radial meshes
 pshead%nmesh=nmesh
 do ii=1,nmesh
   radmesh(ii)%mesh_type=2
   radmesh(ii)%lstep=pawrad%lstep
   radmesh(ii)%mesh_size=meshsz(ii)
   radmesh(ii)%rstep=pawrad%rstep
 end do
 mesh_indx(1)=1
 mesh_indx(2)=iprjmsh
 mesh_indx(3)=icormsh
 mesh_indx(4)=ivlcmsh
 mesh_indx(5)=1
 mesh_indx(6)=ivalmsh
 do ii=1,nmesh
   allocate(radmesh(ii)%rad(radmesh(ii)%mesh_size))
   allocate(radmesh(ii)%radfact(radmesh(ii)%mesh_size))
   allocate(radmesh(ii)%simfact(radmesh(ii)%mesh_size))
   call init_radmesh(radmesh(ii),0.d0)
 end do

!Create ABINIT output
 call wrabinitps(fname,mesh_indx,pawps,pshead,radmesh)

!Create XML output
 call wrxmlps(fname,mesh_indx,pawps,pshead,radmesh)

!Free memory
 do ii=1,nmesh
   deallocate(radmesh(ii)%rad)
   deallocate(radmesh(ii)%radfact)
   deallocate(radmesh(ii)%simfact)
 end do

 end subroutine wrpawps
!!***

! *************************************************************************
! Additional routines and functions
! *************************************************************************

 subroutine init_radmesh(mesh,r_for_intg)
!Initialize arrays in a radial mesh datastructure
 use defs_basis
 use defs_pawps
 implicit none
 real(dp),intent(in) :: r_for_intg
 type(pawrad_type),intent(inout) :: mesh
 integer :: ir,ir_last,isim,ifromr
 real(dp) :: hh
 character(len=500) :: msg
 if (mesh%mesh_type==1) then
   isim=3
   mesh%stepint=mesh%rstep
   mesh%rad(1)=zero;mesh%radfact(1)=one
   do ir=2,mesh%mesh_size
     mesh%rad(ir) =mesh%rstep*dble(ir-1)
     mesh%radfact(ir)=one
   end do
 else if (mesh%mesh_type==2) then
   isim=3
   mesh%stepint=mesh%lstep
   mesh%rad(1)=zero;mesh%radfact(1)=mesh%rstep
   do ir=2,mesh%mesh_size
     mesh%rad(ir) =mesh%rstep*(exp(mesh%lstep*dble(ir-1))-one)
     mesh%radfact(ir)=mesh%rad(ir)+mesh%rstep
   end do
 else if (mesh%mesh_type==3) then
   isim=4
   mesh%stepint=mesh%lstep
   mesh%rad(1)=zero;mesh%radfact(1)=zero
   do ir=2,mesh%mesh_size
     mesh%rad(ir) =mesh%rstep*exp(mesh%lstep*dble(ir-2))
     mesh%radfact(ir)=mesh%rad(ir)
   end do
 else if (mesh%mesh_type==4) then
   isim=3
   mesh%lstep=one/dble(mesh%mesh_size)
   mesh%stepint=mesh%lstep
   mesh%rad(1)=zero;mesh%radfact(1)=mesh%rstep
   do ir=2,mesh%mesh_size
     mesh%rad(ir) =-mesh%rstep*log(one-mesh%lstep*dble(ir-1))
     mesh%radfact(ir)=mesh%rstep/(one-mesh%lstep*dble(ir-1))
   end do
 else if (mesh%mesh_type==5) then
   isim=3
   mesh%stepint=one
   mesh%rad(1)=zero;mesh%radfact(1)=mesh%rstep/mesh%lstep
   do ir=2,mesh%mesh_size
     mesh%rad(ir) =mesh%rstep*dble(ir-1)/(mesh%lstep-dble(ir-1))
     mesh%radfact(ir)=(mesh%rstep+mesh%rad(ir))/(mesh%lstep-dble(ir-1))
   end do
 else !  Other values of mesh_type are not allowed (see psp7in.F90)
  write(msg,'(a,i0)')" Unknow value of mesh_type: ",mesh%mesh_type
  stop
 end if
 mesh%int_meshsz=mesh%mesh_size
 if (r_for_intg>0.d0) then
   ir=min(ifromr(mesh,r_for_intg),mesh%mesh_size)
   if (ir<mesh%mesh_size) then
     if (abs(mesh%rad(ir+1)-r_for_intg)<abs(mesh%rad(ir)-r_for_intg)) ir=ir+1
   end if
   if (ir>1) then
     if (abs(mesh%rad(ir-1)-r_for_intg)<abs(mesh%rad(ir)-r_for_intg)) ir=ir-1
   end if
   mesh%int_meshsz=ir
 end if
 hh=mesh%stepint/3.d0
 mesh%simfact(mesh%int_meshsz)=hh*mesh%radfact(mesh%int_meshsz)
 mesh%simfact(1:isim-2)=zero
 ir_last=1
 do ir=mesh%int_meshsz,isim,-2
   mesh%simfact(ir-1)=4.d0*hh*mesh%radfact(ir-1)
   mesh%simfact(ir-2)=2.d0*hh*mesh%radfact(ir-2)
   ir_last=ir-2
 end do
 mesh%simfact(ir_last)=half*mesh%simfact(ir_last)
 if (mesh%int_meshsz<mesh%mesh_size) mesh%simfact(mesh%int_meshsz+1:mesh%mesh_size)=zero
 end subroutine init_radmesh

! *************************************************************************

 function ifromr(radmesh,rr)
!Return index in a radmesh datastructure from a radius
 use defs_basis
 use defs_pawps
 implicit none
 integer :: ifromr
 real(dp),intent(in) :: rr
 type(pawrad_type),intent(in) :: radmesh
 character(len=500) :: msg
 if (radmesh%mesh_type==1) then
   ifromr=int(tol8+rr/radmesh%rstep)+1
 else if (radmesh%mesh_type==2) then
   ifromr=int(tol8+log(1.d0+rr/radmesh%rstep)/radmesh%lstep)+1
 else if (radmesh%mesh_type==3) then
   if (rr<radmesh%rstep) then
     ifromr=1
   else
     ifromr=int(tol8+log(rr/radmesh%rstep)/radmesh%lstep)+2
   end if
 else if (radmesh%mesh_type==4) then
   ifromr=int(tol8+(1.d0-exp(-rr/radmesh%rstep))/radmesh%lstep)+1
 else if (radmesh%mesh_type==5) then
   ifromr=int(tol8+(radmesh%lstep*rr)/(radmesh%rstep+rr))+1
 else
!  Other values of mesh_type are not allowed (see psp7in.F90)
   write(msg,'(a,i0)')" Unknown value of %mesh_type ",radmesh%mesh_type
   stop
 end if
end function ifromr